use js_sys::{Float32Array, Uint8Array};
use wasm_bindgen::prelude::*;

extern crate web_sys;

// A macro to provide `println!(..)`-style syntax for `console.log` logging.
macro_rules! log {
    ( $( $t:tt )* ) => {
        web_sys::console::log_1(&format!( $( $t )* ).into());
    }
}

#[wasm_bindgen]
pub fn image_filter_wasm(
    js_img: &Uint8Array,
    mask: &Float32Array,
    sigma_spatial: f32,
    sigma_range: f32,
    mut kernel_size: isize,
    width: isize,
    height: isize,
) {
    if kernel_size % 2 != 1 {
        kernel_size += 1;
    };

    let mut mask_dst: Vec<f32> = vec![0.0; (width * height) as usize];

    let _ = libblur::fast_bilateral_filter_f32(
        &mask.to_vec(),
        &mut mask_dst,
        width as u32,
        height as u32,
        kernel_size as u32,
        sigma_spatial,
        sigma_range,
        libblur::FastBlurChannels::Plane,
    );

    let mut img = js_img.to_vec();
    let mut out_img: Vec<u8> = vec![0; (width * height * 4) as usize];

    // blur entire image
    let _ = libblur::fast_gaussian_next(
        &mut img,
        (width * 4) as u32,
        width as u32,
        height as u32,
        20,
        libblur::FastBlurChannels::Channels4,
        libblur::ThreadingPolicy::Single,
        libblur::EdgeMode::Clamp,
    );

    let orig_img = js_img.to_vec();
    // blur image itself
    for x in 0..width {
        for y in 0..height {
            let index = x + y * width;
            let confidence = mask_dst[index as usize];
            for c in 0..4 {
                out_img[index as usize * 4 + c] =
                    (orig_img[index as usize * 4 + c] as f32 * confidence) as u8;
                out_img[index as usize * 4 + c] +=
                    (img[index as usize * 4 + c] as f32 * (1.0 - confidence)) as u8;
            }
        }
    }

    js_img.copy_from(&out_img);
}
